@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Lauri Laanmets (Proekspert AS) <lauri.laanmets@eesti.ee>
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtBluetooth module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 Lauri Laanmets (Proekspert AS) <lauri.laanmets@eesti.ee>
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "android/devicediscoverybroadcastreceiver_p.h"
 #include <QCoreApplication>
@@ -392,9 +356,9 @@ quint8 resolveAndroidMinorClass(QBluetoothDeviceInfo::MajorDeviceClass major, ji
 
 DeviceDiscoveryBroadcastReceiver::DeviceDiscoveryBroadcastReceiver(QObject* parent): AndroidBroadcastReceiver(parent)
 {
-    addAction(valueForStaticField(JavaNames::BluetoothDevice, JavaNames::ActionFound));
-    addAction(valueForStaticField(JavaNames::BluetoothAdapter, JavaNames::ActionDiscoveryStarted));
-    addAction(valueForStaticField(JavaNames::BluetoothAdapter, JavaNames::ActionDiscoveryFinished));
+    addAction(valueForStaticField<QtJniTypes::BluetoothDevice, JavaNames::ActionFound>());
+    addAction(valueForStaticField<QtJniTypes::BluetoothAdapter, JavaNames::ActionDiscoveryStarted>());
+    addAction(valueForStaticField<QtJniTypes::BluetoothAdapter, JavaNames::ActionDiscoveryFinished>());
 }
 
 // Runs in Java thread
@@ -404,35 +368,32 @@ void DeviceDiscoveryBroadcastReceiver::onReceive(JNIEnv *env, jobject context, j
     Q_UNUSED(env)
 
     QJniObject intentObject(intent);
-    const QString action = intentObject.callObjectMethod("getAction", "()Ljava/lang/String;").toString();
+    const QString action = intentObject.callMethod<jstring>("getAction").toString();
 
     qCDebug(QT_BT_ANDROID) << "DeviceDiscoveryBroadcastReceiver::onReceive() - event:" << action;
 
-    if (action == valueForStaticField(JavaNames::BluetoothAdapter,
-                                      JavaNames::ActionDiscoveryFinished).toString()) {
+    if (action == valueForStaticField<QtJniTypes::BluetoothAdapter,
+                                      JavaNames::ActionDiscoveryFinished>().toString()) {
         emit finished();
-    } else if (action == valueForStaticField(JavaNames::BluetoothAdapter,
-                                             JavaNames::ActionDiscoveryStarted).toString()) {
-
-    } else if (action == valueForStaticField(JavaNames::BluetoothDevice,
-                                             JavaNames::ActionFound).toString()) {
+    } else if (action == valueForStaticField<QtJniTypes::BluetoothAdapter,
+                                             JavaNames::ActionDiscoveryStarted>().toString()) {
+        emit discoveryStarted();
+    } else if (action == valueForStaticField<QtJniTypes::BluetoothDevice,
+                                             JavaNames::ActionFound>().toString()) {
         //get BluetoothDevice
-        QJniObject keyExtra = valueForStaticField(JavaNames::BluetoothDevice,
-                                                         JavaNames::ExtraDevice);
+        QJniObject keyExtra = valueForStaticField<QtJniTypes::BluetoothDevice,
+                                                  JavaNames::ExtraDevice>();
         const QJniObject bluetoothDevice =
-                intentObject.callObjectMethod("getParcelableExtra",
-                                              "(Ljava/lang/String;)Landroid/os/Parcelable;",
-                                              keyExtra.object<jstring>());
+                intentObject.callMethod<QtJniTypes::Parcelable>("getParcelableExtra",
+                                                                keyExtra.object<jstring>());
 
         if (!bluetoothDevice.isValid())
             return;
 
-        keyExtra = valueForStaticField(JavaNames::BluetoothDevice,
-                                       JavaNames::ExtraRssi);
+        keyExtra = valueForStaticField<QtJniTypes::BluetoothDevice,
+                                       JavaNames::ExtraRssi>();
         int rssi = intentObject.callMethod<jshort>("getShortExtra",
-                                                "(Ljava/lang/String;S)S",
-                                                keyExtra.object<jstring>(),
-                                                0);
+                                                   keyExtra.object<jstring>(), jshort(0));
 
         const QBluetoothDeviceInfo info = retrieveDeviceInfo(bluetoothDevice, rssi);
         if (info.isValid())
@@ -455,11 +416,12 @@ void DeviceDiscoveryBroadcastReceiver::onReceiveLeScan(
 
 QBluetoothDeviceInfo DeviceDiscoveryBroadcastReceiver::retrieveDeviceInfo(const QJniObject &bluetoothDevice, int rssi, jbyteArray scanRecord)
 {
-    const QString deviceName = bluetoothDevice.callObjectMethod<jstring>("getName").toString();
-    const QBluetoothAddress deviceAddress(bluetoothDevice.callObjectMethod<jstring>("getAddress").toString());
+    const QString deviceName = bluetoothDevice.callMethod<jstring>("getName").toString();
+    const QBluetoothAddress deviceAddress(
+                bluetoothDevice.callMethod<jstring>("getAddress").toString());
+    const QJniObject bluetoothClass =
+            bluetoothDevice.callMethod<QtJniTypes::BluetoothClass>("getBluetoothClass");
 
-    const QJniObject bluetoothClass = bluetoothDevice.callObjectMethod("getBluetoothClass",
-                                                                        "()Landroid/bluetooth/BluetoothClass;");
     if (!bluetoothClass.isValid())
         return QBluetoothDeviceInfo();
 
@@ -489,7 +451,7 @@ QBluetoothDeviceInfo DeviceDiscoveryBroadcastReceiver::retrieveDeviceInfo(const
     quint32 serviceResult = 0;
     for (quint32 current : services) {
         int androidId = (current << 16); // Android values shift by 2 bytes compared to Qt enums
-        if (bluetoothClass.callMethod<jboolean>("hasService", "(I)Z", androidId))
+        if (bluetoothClass.callMethod<jboolean>("hasService", androidId))
             serviceResult |= current;
     }
 
