@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtBluetooth module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <QtCore/QGlobalStatic>
 #include <QtCore/QLoggingCategory>
@@ -50,6 +14,12 @@
 
 QT_BEGIN_NAMESPACE
 
+QT_IMPL_METATYPE_EXTERN(InterfaceList)
+QT_IMPL_METATYPE_EXTERN(ManufacturerDataList)
+QT_IMPL_METATYPE_EXTERN(ServiceDataList)
+QT_IMPL_METATYPE_EXTERN(ManagedObjectList)
+
+
 Q_DECLARE_LOGGING_CATEGORY(QT_BT_BLUEZ)
 
 typedef enum Bluez5TestResultType
@@ -62,7 +32,7 @@ typedef enum Bluez5TestResultType
 Q_GLOBAL_STATIC_WITH_ARGS(Bluez5TestResult, bluezVersion, (BluezVersionUnknown))
 Q_GLOBAL_STATIC_WITH_ARGS(QVersionNumber, bluezDaemonVersion, (QVersionNumber()))
 
-/*!
+/*
     Ensures that the DBus types are registered
  */
 
@@ -75,6 +45,7 @@ void initializeBluez5()
         qDBusRegisterMetaType<InterfaceList>();
         qDBusRegisterMetaType<ManagedObjectList>();
         qDBusRegisterMetaType<ManufacturerDataList>();
+        qDBusRegisterMetaType<ServiceDataList>();
 
         QDBusPendingReply<ManagedObjectList> reply = manager.GetManagedObjects();
         reply.waitForFinished();
@@ -166,7 +137,7 @@ bool mandatoryHciIoctlsAvailable()
     return true;
 }
 
-/*!
+/*
  * This function returns the version of bluetoothd in use on the system.
  * This is required to determine which QLEControllerPrivate implementation
  * is required. The following version tags are of significance:
@@ -190,6 +161,7 @@ QVersionNumber bluetoothdVersion()
         qDBusRegisterMetaType<InterfaceList>();
         qDBusRegisterMetaType<ManagedObjectList>();
         qDBusRegisterMetaType<ManufacturerDataList>();
+        qDBusRegisterMetaType<ServiceDataList>();
 
         qCDebug(QT_BT_BLUEZ) << "Detecting bluetoothd version";
         //Order of matching
@@ -475,7 +447,7 @@ void QtBluezDiscoveryManager::removeAdapterFromMonitoring(const QString &dbusPat
     emit discoveryInterrupted(dbusPath);
 }
 
-/*!
+/*
     Finds the path for the local adapter with \a wantedAddress or an empty string
     if no local adapter with the given address can be found.
     If \a wantedAddress is \c null it returns the first/default adapter or an empty
@@ -531,7 +503,7 @@ QString findAdapterForAddress(const QBluetoothAddress &wantedAddress, bool *ok =
     if (wantedAddress.isNull())
         return localAdapters.front().first; // -> return first found adapter
 
-    for (const AddressForPathType &pair : qAsConst(localAdapters)) {
+    for (const AddressForPathType &pair : std::as_const(localAdapters)) {
         if (pair.second == wantedAddress)
             return pair.first; // -> found local adapter with wanted address
     }
@@ -547,7 +519,7 @@ QString findAdapterForAddress(const QBluetoothAddress &wantedAddress, bool *ok =
 QString sanitizeNameForDBus(const QString &text)
 {
     QString appName = text;
-    for (int i = 0; i < appName.length(); i++) {
+    for (qsizetype i = 0; i < appName.size(); ++i) {
         ushort us = appName[i].unicode();
         bool valid = (us >= 'a' && us <= 'z')
                       || (us >= 'A' && us <= 'Z')
@@ -562,3 +534,5 @@ QString sanitizeNameForDBus(const QString &text)
 }
 
 QT_END_NAMESPACE
+
+#include "moc_bluez5_helper_p.cpp"
