@@ -1,47 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtBluetooth module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qbluetoothservicediscoveryagent_p.h"
 #include "qbluetoothsocket_android_p.h"
 #include "android/servicediscoverybroadcastreceiver_p.h"
 #include "android/localdevicebroadcastreceiver_p.h"
 #include "android/androidutils_p.h"
+#include "android/jni_android_p.h"
 
 #include <QCoreApplication>
 #include <QtCore/qcoreapplication.h>
@@ -56,6 +21,8 @@ QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(QT_BT_ANDROID)
 
+static constexpr auto uuidFetchTimeLimit = std::chrono::seconds{4};
+
 QBluetoothServiceDiscoveryAgentPrivate::QBluetoothServiceDiscoveryAgentPrivate(
         QBluetoothServiceDiscoveryAgent *qp, const QBluetoothAddress &deviceAdapter)
     : error(QBluetoothServiceDiscoveryAgent::NoError),
@@ -92,9 +59,8 @@ QBluetoothServiceDiscoveryAgentPrivate::QBluetoothServiceDiscoveryAgentPrivate(
     */
 
     if (createAdapter)
-        btAdapter = QJniObject::callStaticObjectMethod("android/bluetooth/BluetoothAdapter",
-                                                           "getDefaultAdapter",
-                                                           "()Landroid/bluetooth/BluetoothAdapter;");
+        btAdapter = getDefaultBluetoothAdapter();
+
     if (!btAdapter.isValid())
         qCWarning(QT_BT_ANDROID) << "Platform does not support Bluetooth";
 
@@ -119,8 +85,9 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
 
     if (!ensureAndroidPermission(BluetoothPermission::Connect)) {
         qCWarning(QT_BT_ANDROID) << "Service discovery start() failed due to missing permissions";
-        error = QBluetoothServiceDiscoveryAgent::UnknownError;
-        errorString = QBluetoothServiceDiscoveryAgent::tr("Unable to perform SDP scan");
+        error = QBluetoothServiceDiscoveryAgent::MissingPermissionsError;
+        errorString = QBluetoothServiceDiscoveryAgent::tr(
+                "Failed to start service discovery due to missing permissions.");
         emit q->errorOccurred(error);
         _q_serviceDiscoveryFinished();
         return;
@@ -147,9 +114,8 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
 
     QJniObject inputString = QJniObject::fromString(address.toString());
     QJniObject remoteDevice =
-            btAdapter.callObjectMethod("getRemoteDevice",
-                                               "(Ljava/lang/String;)Landroid/bluetooth/BluetoothDevice;",
-                                               inputString.object<jstring>());
+            btAdapter.callMethod<QtJniTypes::BluetoothDevice>("getRemoteDevice",
+                                                              inputString.object<jstring>());
     if (!remoteDevice.isValid()) {
 
         //if it was only device then its error -> otherwise go to next device
@@ -171,8 +137,8 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
                                << ")" << address.toString() ;
 
         //Minimal discovery uses BluetoothDevice.getUuids()
-        QJniObject parcelUuidArray = remoteDevice.callObjectMethod(
-                    "getUuids", "()[Landroid/os/ParcelUuid;");
+        QJniObject parcelUuidArray =
+                remoteDevice.callMethod<QtJniTypes::ParcelUuidArray>("getUuids");
 
         if (!parcelUuidArray.isValid()) {
             if (singleDevice) {
@@ -251,9 +217,9 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_processFetchedUuids(
     if (address.isNull() || uuids.isEmpty()) {
         if (discoveredDevices.size() == 1) {
             Q_Q(QBluetoothServiceDiscoveryAgent);
-            QTimer::singleShot(4000, q, [this]() {
+            QTimer::singleShot(uuidFetchTimeLimit, q, [this]() {
                 this->_q_fetchUuidsTimeout();
-            }); // will also call _q_seriveDiscoveryFinished()
+            }); // will also call _q_serviceDiscoveryFinished()
         } else {
             _q_serviceDiscoveryFinished();
         }
@@ -270,7 +236,7 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_processFetchedUuids(
         qCDebug(QT_BT_ANDROID) << result;
     }
 
-    /* In general there are two uuid events per device.
+    /* In general there may be up-to two uuid events per device.
      * We'll wait for the second event to arrive before we process the UUIDs.
      * We utilize a timeout to catch cases when the second
      * event doesn't arrive at all.
@@ -301,10 +267,10 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_processFetchedUuids(
         sdpCache.insert(address, pair);
 
         //the discovery on the last device cannot immediately finish
-        //we have to grant the 2 seconds timeout delay
+        //we have to grant the timeout delay to allow potential second event to arrive
         if (discoveredDevices.size() == 1) {
             Q_Q(QBluetoothServiceDiscoveryAgent);
-            QTimer::singleShot(4000, q, [this]() {
+            QTimer::singleShot(uuidFetchTimeLimit, q, [this]() {
                 this->_q_fetchUuidsTimeout();
             });
             return;
@@ -434,7 +400,7 @@ void QBluetoothServiceDiscoveryAgentPrivate::populateDiscoveredServices(const QB
         if (!uuidFilter.isEmpty()) {
             bool match = uuidFilter.contains(serviceInfo.serviceUuid());
             match |= uuidFilter.contains(QBluetoothSocketPrivateAndroid::reverseUuid(serviceInfo.serviceUuid()));
-            for (const auto &uuid : qAsConst(uuidFilter)) {
+            for (const auto &uuid : std::as_const(uuidFilter)) {
                 match |= serviceInfo.serviceClassUuids().contains(uuid);
                 match |= serviceInfo.serviceClassUuids().contains(QBluetoothSocketPrivateAndroid::reverseUuid(uuid));
             }
@@ -457,14 +423,18 @@ void QBluetoothServiceDiscoveryAgentPrivate::populateDiscoveredServices(const QB
 
 void QBluetoothServiceDiscoveryAgentPrivate::_q_fetchUuidsTimeout()
 {
-    if (sdpCache.isEmpty())
+    // In practice if device list is empty, discovery has been stopped or bluetooth is offline
+    if (discoveredDevices.isEmpty())
         return;
 
-    QPair<QBluetoothDeviceInfo,QList<QBluetoothUuid> > pair;
-    const QList<QBluetoothAddress> keys = sdpCache.keys();
-    for (const QBluetoothAddress &key : keys) {
-        pair = sdpCache.take(key);
-        populateDiscoveredServices(pair.first, pair.second);
+    // Process remaining services in the cache (these didn't get a second UUID event)
+    if (!sdpCache.isEmpty()) {
+        QPair<QBluetoothDeviceInfo,QList<QBluetoothUuid> > pair;
+        const QList<QBluetoothAddress> keys = sdpCache.keys();
+        for (const QBluetoothAddress &key : keys) {
+            pair = sdpCache.take(key);
+            populateDiscoveredServices(pair.first, pair.second);
+        }
     }
 
     Q_ASSERT(sdpCache.isEmpty());
