@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtBluetooth module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2017 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qbluetoothserviceinfo.h"
 #include "qbluetoothserviceinfo_p.h"
@@ -130,7 +94,7 @@ qint64 getLengthForBaseType(unsigned char type, ComPtr<IDataReader> &reader)
 bool writeStringHelper(const QString &string, ComPtr<IDataWriter> writer)
 {
     HRESULT hr;
-    const int stringLength = string.length();
+    const qsizetype stringLength = string.size();
     unsigned char type = TYPE_STRING_BASE;
     if (stringLength < 0) {
         qCWarning(QT_BT_WINDOWS) << "Can not write invalid string value to buffer";
@@ -158,7 +122,7 @@ bool writeStringHelper(const QString &string, ComPtr<IDataWriter> writer)
     quint32 bytesWritten;
     hr = writer->WriteString(stringRef.Get(), &bytesWritten);
     RETURN_FALSE_IF_FAILED("Could not write string to buffer.");
-    if (bytesWritten != string.length()) {
+    if (bytesWritten != size_t(string.size())) {
         qCWarning(QT_BT_WINDOWS) << "Did not write full value to buffer";
         return false;
     }
@@ -420,10 +384,12 @@ static ComPtr<IBuffer> bufferFromAttribute(const QVariant &attribute)
 QBluetoothServiceInfoPrivate::QBluetoothServiceInfoPrivate()
     : registered(false)
 {
+    mainThreadCoInit(this);
 }
 
 QBluetoothServiceInfoPrivate::~QBluetoothServiceInfoPrivate()
 {
+    mainThreadCoUninit(this);
 }
 
 bool QBluetoothServiceInfoPrivate::isRegistered() const
