@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2018 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtBluetooth module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2018 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qbluetoothsocket.h"
 #include "qbluetoothsocket_bluezdbus_p.h"
@@ -280,10 +244,14 @@ void QBluetoothSocketPrivateBluezDBus::connectToService(
         qCWarning(QT_BT_BLUEZ) << "Cannot find appropriate serviceUuid"
                                << "or SerialPort service class uuid";
         errorString = QBluetoothSocket::tr("Missing serviceUuid or Serial Port service class uuid");
-        q->setSocketError(QBluetoothSocket::SocketError::OperationError);
+        q->setSocketError(QBluetoothSocket::SocketError::UnsupportedProtocolError);
         return;
     }
 
+    if (service.socketProtocol() != QBluetoothServiceInfo::Protocol::UnknownProtocol)
+        socketType = service.socketProtocol();
+    qCDebug(QT_BT_BLUEZ) << "Socket protocol used:" << socketType;
+
     connectToService(service.device().address(), targetService, openMode);
 }
 
@@ -295,7 +263,7 @@ void QBluetoothSocketPrivateBluezDBus::connectToService(
     if (address.isNull()) {
         qCWarning(QT_BT_BLUEZ) << "Invalid address to remote address passed.";
         errorString = QBluetoothSocket::tr("Invalid Bluetooth address passed to connectToService()");
-        q->setSocketError(QBluetoothSocket::SocketError::OperationError);
+        q->setSocketError(QBluetoothSocket::SocketError::UnsupportedProtocolError);
         return;
     }
 
@@ -303,7 +271,7 @@ void QBluetoothSocketPrivateBluezDBus::connectToService(
         qCWarning(QT_BT_BLUEZ) << "Cannot find appropriate serviceUuid"
                                << "or SerialPort service class uuid";
         errorString = QBluetoothSocket::tr("Missing serviceUuid or Serial Port service class uuid");
-        q->setSocketError(QBluetoothSocket::SocketError::OperationError);
+        q->setSocketError(QBluetoothSocket::SocketError::UnsupportedProtocolError);
         return;
     }
 
@@ -623,3 +591,5 @@ void QBluetoothSocketPrivateBluezDBus::clearSocket()
     profilePath.clear();
 }
 QT_END_NAMESPACE
+
+#include "moc_qbluetoothsocket_bluezdbus_p.cpp"
